"""A modification workflow for migrating an ImportedGeantIP to an GeantIP subscription."""

from orchestrator.targets import Target
from orchestrator.types import State, UUIDstr
from orchestrator.utils.errors import ProcessFailureError
from orchestrator.workflow import StepList, done, init, step, workflow
from orchestrator.workflows.steps import resync, store_process_subscription, unsync
from orchestrator.workflows.utils import wrap_modify_initial_input_form

from gso.products import ProductName
from gso.products.product_types.l3_core_service import (
    ImportedL3CoreService,
    L3CoreService,
    L3CoreServiceType,
)
from gso.services.subscriptions import get_product_id_by_name


@step("Create imported subscription")
def import_l3_core_service_subscription(subscription_id: UUIDstr) -> State:
    """Take an imported subscription, and turn it into an L3 Core Service subscription."""
    old_l3_core_service = ImportedL3CoreService.from_subscription(subscription_id)
    match old_l3_core_service.l3_core_service_type:
        case L3CoreServiceType.IMPORTED_GEANT_IP:
            new_subscription_id = get_product_id_by_name(ProductName.GEANT_IP)
        case L3CoreServiceType.IMPORTED_IAS:
            new_subscription_id = get_product_id_by_name(ProductName.IAS)
        case L3CoreServiceType.IMPORTED_GWS:
            new_subscription_id = get_product_id_by_name(ProductName.GWS)
        case L3CoreServiceType.IMPORTED_LHCONE:
            new_subscription_id = get_product_id_by_name(ProductName.LHCONE)
        case L3CoreServiceType.IMPORTED_COPERNICUS:
            new_subscription_id = get_product_id_by_name(ProductName.COPERNICUS)
        case _:
            msg = f"This {old_l3_core_service.l3_core_service_type} is already imported, nothing to do."
            raise ProcessFailureError(message=msg, details=old_l3_core_service)
    new_subscription = L3CoreService.from_other_product(old_l3_core_service, new_subscription_id)  # type: ignore[arg-type]

    return {"subscription": new_subscription}


@workflow("Import L3 Core Service", target=Target.MODIFY, initial_input_form=wrap_modify_initial_input_form(None))
def import_l3_core_service() -> StepList:
    """Modify an imported subscription into an L3 Core Service subscription to complete the import."""
    return (
        init
        >> store_process_subscription(Target.MODIFY)
        >> unsync
        >> import_l3_core_service_subscription
        >> resync
        >> done
    )
